@@ -1,6 +1,6 @@
 /*
   A widget to display and manipulate tabular data
-  Copyright (C) 2016, 2017  John Darrington
+  Copyright (C) 2016, 2017, 2019, 2020  John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -101,6 +101,17 @@ struct _SswSheetAxisPrivate
 
 typedef struct _SswSheetAxisPrivate SswSheetAxisPrivate;
 
+static void
+axis_debug (const char *fmt, ...)
+{
+#if 0
+  va_list args;
+  va_start (args, fmt);
+  g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, fmt, args);
+  va_end (args);
+#endif
+}
+
 enum  {CHANGED,
        HEADER_CLICKED,
        HEADER_DOUBLE_CLICKED,
@@ -469,10 +480,10 @@ item_start (SswSheetAxis *axis, guint index)
   return y;
 }
 
-static gint
+static gdouble
 estimated_widget_size (SswSheetAxis *axis)
 {
-  gint avg_widget_size = 0;
+  gdouble avg_widget_size = 0;
   PRIV_DECL (axis);
 
   Foreach_Item avg_widget_size += item_size (axis, item);
@@ -513,7 +524,7 @@ static gint
 estimated_list_size (SswSheetAxis *axis,
                      guint *start_part, guint *end_part)
 {
-  gint avg_size;
+  gdouble avg_size;
   gint start_widgets;
   gint end_widgets;
   gint exact_size = 0;
@@ -606,7 +617,7 @@ configure_adjustment (SswSheetAxis *axis)
 
   if ((gint) cur_upper != MAX (list_size, widget_size))
     {
-      /*g_message ("New upper: %d (%d, %d)", MAX (list_size, widget_size), list_size, widget_size); */
+      axis_debug ("%p New upper: %d (%d, %d)", priv, MAX (list_size, widget_size), list_size, widget_size);
       gtk_adjustment_set_upper (priv->adjustment,
                                 MAX (list_size, widget_size));
     }
@@ -644,10 +655,12 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
 
   /* This "out of sight" case happens when the new value is so different from the old one
    * that we rather just remove all widgets and adjust the model_from/model_to values
+   * This mode only works when some widgets are already there to estimate the widget size
    */
-  if (bin_start (axis) + bin_size < 0 || bin_start (axis) >= widget_size || force_reload)
+  if (priv->widgets->len > 0 && bin_size > 0 &&
+      (bin_start (axis) + bin_size < 0 || bin_start (axis) >= widget_size || force_reload))
     {
-      gint avg_widget_size = estimated_widget_size (axis);
+      gdouble avg_widget_size = estimated_widget_size (axis);
       gdouble percentage;
       gdouble value = __axis_get_value (axis);
       gdouble upper = gtk_adjustment_get_upper (priv->adjustment);
@@ -655,8 +668,6 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
       guint start_widget_index;
       gint i;
 
-      /* g_message ("OUT OF SIGHT"); */
-
       for (i = priv->widgets->len - 1; i >= 0; i--)
         remove_child_ginternal (axis, g_ptr_array_index (priv->widgets, i));
       bin_size = 0;             /* The window is empty now, obviously */
@@ -704,7 +715,7 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
         gint w_size = item_size (axis, w);
         if (bin_start (axis) + item_start (axis, i) + w_size < 0)
           {
-            /* g_message ("Removing start widget %d", i); */
+            axis_debug ("%p Removing start widget %d", priv, i);
             priv->bin_start_diff += w_size;
             bin_size -= w_size;
             remove_child_ginternal (axis, w);
@@ -730,7 +741,7 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
         priv->bin_start_diff -= min;
         bin_size += min;
         start_added = TRUE;
-        /* g_message ("Adding start widget for index %d", priv->model_from); */
+        axis_debug ("%p Adding start widget for index %d", priv, priv->model_from);
       }
   }
 
@@ -743,10 +754,10 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
         GtkWidget *w = g_ptr_array_index (priv->widgets, i);
         gint y = bin_start (axis) + item_start (axis, i);
 
-        /*g_message ("%d: %d + %d > %d", i, bin_start (axis), item_start (axis, i), widget_size); */
+        axis_debug ("%p %d: %d + %d > %d", priv, i, bin_start (axis), item_start (axis, i), widget_size);
         if (y > widget_size)
           {
-            /*g_message ("Removing widget %d", i); */
+            axis_debug ("%p Removing widget %d", priv, i);
             gint w_size = item_size (axis, w);
             remove_child_ginternal (axis, w);
             bin_size -= w_size;
@@ -761,14 +772,14 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
   /* Insert end widgets */
   {
     end_added = FALSE;
-    /*g_message ("%d + %d <= %d", bin_start (axis), bin_size, widget_size); */
+    axis_debug ("%p %d + %d <= %d", priv, bin_start (axis), bin_size, widget_size);
     while (bin_start (axis) + bin_size <= widget_size &&
            priv->model_to < ssw_sheet_axis_get_extent (axis))
       {
         GtkWidget *new_widget;
         gint min;
 
-        /* g_message ("Inserting end widget for position %u at %u", priv->model_to, priv->widgets->len); */
+        axis_debug ("%p Inserting end widget for position %u at %u", priv, priv->model_to, priv->widgets->len);
         new_widget = get_widget (axis, priv->model_to);
         insert_child_ginternal (axis, new_widget, priv->widgets->len);
         min = item_size (axis, new_widget);
@@ -795,18 +806,18 @@ ensure_visible_widgets (SswSheetAxis *axis, gboolean force_reload)
 
     configure_adjustment (axis);
 
-    /*g_message ("Setting value to %f - %d", priv->bin_start_diff, bin_window_y); */
+    axis_debug ("%p Setting value to %f - %d", priv, priv->bin_start_diff, bin_window_y);
     __axis_set_value (axis,
                               priv->bin_start_diff - bin_window_y);
     if (__axis_get_value (axis) < priv->bin_start_diff)
       {
         __axis_set_value (axis, priv->bin_start_diff);
-        /*g_message ("Case 1"); */
+        axis_debug ("%p Case 1", priv);
       }
 
     if (bin_start (axis) > 0)
       {
-        /*g_message ("CRAP"); */
+        axis_debug ("%p CRAP", priv);
         priv->bin_start_diff = __axis_get_value (axis);
       }
   }
@@ -838,7 +849,7 @@ items_changed_cb (GListModel *model,
 
   if (! gtk_widget_get_realized (GTK_WIDGET (axis)))
     return;
-  
+
   /* If the change is out of our visible range anyway,
      then we don't care. */
   if (position > priv->model_to && bin_window_full (axis))
@@ -905,6 +916,7 @@ __size_allocate (GtkWidget *widget, GtkAllocation *allocation)
                               allocation->width, allocation->height);
 
       update_bin_window ((SswSheetAxis *) widget);
+      ensure_visible_widgets ((SswSheetAxis *) widget, FALSE);
     }
 
 
@@ -1263,13 +1275,6 @@ ssw_sheet_axis_new (GtkOrientation orientation)
 				   NULL));
 }
 
-static void
-force_update (SswSheetAxis *axis)
-{
-  ensure_visible_widgets (axis, TRUE);
-  gtk_widget_queue_draw (GTK_WIDGET (axis));
-}
-
 void
 ssw_sheet_axis_set_model (SswSheetAxis *axis, GListModel *model)
 {
@@ -1286,8 +1291,6 @@ ssw_sheet_axis_set_model (SswSheetAxis *axis, GListModel *model)
       g_object_ref (model);
     }
 
-  g_signal_connect_object (model, "notify", G_CALLBACK (force_update),
-			   axis, G_CONNECT_SWAPPED);
   ensure_visible_widgets (axis, TRUE);
 }
 
@@ -1330,7 +1333,7 @@ ssw_sheet_axis_class_init (SswSheetAxisClass *class)
 			 P_("The Adjustment"),
 			 GTK_TYPE_ADJUSTMENT,
 			 G_PARAM_READWRITE);
-  
+
   GParamSpec *draggable_spec =
     g_param_spec_boolean ("draggable",
 			  P_("Draggable"),
@@ -1437,7 +1440,7 @@ ssw_sheet_axis_class_init (SswSheetAxisClass *class)
   g_object_class_install_property (object_class,
                                    PROP_DRAGGABLE,
                                    draggable_spec);
-  
+
   g_object_class_override_property (object_class, PROP_ORIENTATION,
                                     "orientation");
 }
@@ -1632,7 +1635,7 @@ ssw_sheet_axis_override_size (SswSheetAxis *axis, gint pos, gint size)
 
   if (done_resize)
     return;
-  
+
   g_hash_table_insert (priv->size_override,
 		       GINT_TO_POINTER (pos),
 		       GINT_TO_POINTER (size));
@@ -1755,7 +1758,7 @@ ssw_sheet_axis_get_extent (SswSheetAxis *axis)
 {
   PRIV_DECL (axis);
 
-  gint avg_widget_size = estimated_widget_size (axis);
+  gdouble avg_widget_size = estimated_widget_size (axis);
   if (avg_widget_size == 0)
     avg_widget_size = 28;
 
@@ -2001,4 +2004,3 @@ ssw_sheet_axis_jump_end (SswSheetAxis *axis, gint whereto)
 {
   ssw_sheet_axis_jump_end_with_offset (axis, whereto, 0);
 }
-
