@@ -1,6 +1,6 @@
 /*
   A widget to display and manipulate tabular data
-  Copyright (C) 2016, 2017  John Darrington
+  Copyright (C) 2016, 2017, 2020  John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -879,8 +879,9 @@ __key_press_event (GtkWidget *w, GdkEventKey *e)
 {
   SswSheetBody *body = SSW_SHEET_BODY (w);
   PRIV_DECL (body);
-
-  if (GTK_WIDGET_CLASS (ssw_sheet_body_parent_class)->key_press_event (w, e))
+  /* If send_event is true, then this event has been generated by on_entry_activate*/
+  if ( !e->send_event &&
+       GTK_WIDGET_CLASS (ssw_sheet_body_parent_class)->key_press_event (w, e))
     return TRUE;
 
   if (is_printable_key (e->keyval))
@@ -2204,9 +2205,11 @@ static void
 on_entry_activate (GtkEntry *e, gpointer ud)
 {
   SswSheetBody *body = SSW_SHEET_BODY (ud);
-
+  GdkWindow *win = gtk_widget_get_window (GTK_WIDGET (body));
+  g_return_if_fail (GDK_IS_WINDOW (win));
   GdkEvent *ev = gdk_event_new (GDK_KEY_PRESS);
   GdkEventKey *event = (GdkEventKey *) ev;
+  event->window = g_object_ref(win);
   event->keyval = GDK_KEY_Return;
   event->send_event = TRUE;
   gboolean ret;
